@@ -301,7 +301,11 @@ class DB2TypeCompiler(compiler.GenericTypeCompiler):
         return self.visit_FLOAT(type_)
 
     def visit_unicode(self, type_):
-        return self.visit_VARGRAPHIC(type_)
+        check_server = getattr(DB2Dialect,'serverType')
+        if(check_server == "DB2"):
+            return (self.visit_VARGRAPHIC(type_) + " CCSID 1200")
+        else:
+            return self.visit_VARGRAPHIC(type_)
 
     def visit_unicode_text(self, type_):
         return self.visit_LONGVARGRAPHIC(type_)
@@ -436,7 +440,7 @@ class DB2Compiler(compiler.SQLCompiler):
         # for example
         if isinstance(type_, (
                     sa_types.DateTime, sa_types.Date, sa_types.Time,
-                    sa_types.DECIMAL)):
+                    sa_types.DECIMAL,sa_types.String)):
             return super(DB2Compiler, self).visit_cast(cast, **kw)
         else:
             return self.process(cast.clause)
@@ -613,18 +617,10 @@ class DB2IdentifierPreparer(compiler.IdentifierPreparer):
     reserved_words = RESERVED_WORDS
     illegal_initial_characters = set(range(0, 10)).union(["_", "$"])
 
-
-class DB2ExecutionContext(default.DefaultExecutionContext):
-    def fire_sequence(self, seq, type_):
-        return self._execute_scalar("SELECT NEXTVAL FOR " +
-                    self.dialect.identifier_preparer.format_sequence(seq) +
-                    " FROM SYSIBM.SYSDUMMY1", type_)
-
 class _SelectLastRowIDMixin(object):
     _select_lastrowid = False
     _lastrowid = None
 
-
     def get_lastrowid(self):
         return self._lastrowid
 
@@ -649,6 +645,13 @@ class _SelectLastRowIDMixin(object):
                 self._lastrowid = int(row[0])
 
 
+class DB2ExecutionContext(_SelectLastRowIDMixin,default.DefaultExecutionContext):
+    def fire_sequence(self, seq, type_):
+        return self._execute_scalar("SELECT NEXTVAL FOR " +
+                    self.dialect.identifier_preparer.format_sequence(seq) +
+                    " FROM SYSIBM.SYSDUMMY1", type_)
+
+
 class DB2Dialect(default.DefaultDialect):
 
     name = 'ibm_db_sa'
@@ -660,7 +663,10 @@ class DB2Dialect(default.DefaultDialect):
     supports_char_length = False
     supports_unicode_statements = False
     supports_unicode_binds = False
-    returns_unicode_strings = False
+    if SA_Version < [1,4]:
+        returns_unicode_strings = False
+    else:
+        returns_unicode_strings = sa_types.String.RETURNS_CONDITIONAL
     postfetch_lastrowid = True
     supports_sane_rowcount = True
     supports_sane_multi_rowcount = True
@@ -686,6 +692,7 @@ class DB2Dialect(default.DefaultDialect):
     execution_ctx_cls = DB2ExecutionContext
 
     _reflector_cls = ibm_reflection.DB2Reflector
+    serverType = ''
 
     def __init__(self, **kw):
         super(DB2Dialect, self).__init__(**kw)
@@ -695,9 +702,24 @@ class DB2Dialect(default.DefaultDialect):
     # reflection: these all defer to an BaseDB2Reflector
     # object which selects between DB2 and AS/400 schemas
     def initialize(self, connection):
-        super(DB2Dialect, self).initialize(connection)
         self.dbms_ver = getattr(connection.connection, 'dbms_ver', None)
         self.dbms_name = getattr(connection.connection, 'dbms_name', None)
+        DB2Dialect.serverType = self.dbms_name
+        super(DB2Dialect, self).initialize(connection)
+        #check server type logic here
+        if(self.dbms_name == 'AS'):
+            _reflector_cls = ibm_reflection.AS400Reflector
+        elif(self.dbms_name == "DB2"):
+            _reflector_cls = ibm_reflection.OS390Reflector
+        elif(self.dbms_name is None):
+            _reflector_cls = ibm_reflection.DB2Reflector
+        elif("DB2/" in self.dbms_name):
+            _reflector_cls = ibm_reflection.DB2Reflector
+        elif("IDS/" in self.dbms_name):
+            _reflector_cls = ibm_reflection.DB2Reflector
+        
+        self._reflector = _reflector_cls(self) 
+
         
     def normalize_name(self, name):
         return self._reflector.normalize_name(name)
